/*
--------------------------------------------------------------------------------
Title:              AmpEd_ArduinoGUI_test.ino
Description:        The testing version of the skeleton, for functionality. This
                    one is updated and worked on.

Notes:              "Command signal" codes:
                      'J' = regular joystick
                      'U' = unidirectional joystick
                      'H' = head switches
                      'S' = start recording
                      'D' = done recording

                    4/7/2020: Moved some code around and deleted unnecessary
                    and functions that don't work as intended.
                    Update: had to change the schema a little bit, but now this
                    code can differentiate between the "preset" and
                    "start/stop recording" signals. The presets were taken care
                    of in the while loop on the setup() function.
                    
                      
Author:             Taylor Yee
Created:            04/07/2020
Last Modified:      04/07/2020
Helpful Link(s):    https://forum.arduino.cc/index.php?topic=447906.0
--------------------------------------------------------------------------------
*/
const int trigPin = 9;
const int echoPin = 10;
float duration, distance;

// String representation of data collected - used when session ends
String results = "";

// Sample variables to hold data needed
int numCollisions;
int closestDistance;

// Character variables to hold "command signals" from Processing
char presetInput;
char doneRecording;

void setup()
{
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  
  // Put all your other general setup here (such as sensor pin setup, etc.)
  Serial.begin(9600);

  numCollisions = 0;
  closestDistance = 0;

  while(!Serial.available())
  {
    
  }
  
  if(Serial.available() == 'U' || Serial.available() == 'J' || Serial.available() == 'H')
  {
    presetInput = Serial.read();
  }

  else
  {
    presetInput = Serial.read();
  }
  
  // If regular joystick preset was chosen
  if(presetInput == 'J')
  {
    /** 
     * Could write setup code raw here. If you want to do that, then
     * delete the function call below
     */
     regJoystickSetup();
  }

  // If unidirectional joystick preset was chosen
  else if(presetInput == 'U')
  {
    /** 
     * Could write setup code raw here. If you want to do that, then
     * delete the function call below
     */
     uniJoystickSetup();
  }

  // If head switch preset was chosen
  else if(presetInput == 'H')
  {
    /** 
     * Could write setup code raw here. If you want to do that, then
     * delete the function call below
     */
     headSwitchSetup();
  }
}

void loop()
{
  /**
   * DO NOT DELETE THIS IF STATEMENT. This allows us to go back and change settings
   * in the setup() function above as needed.
   */
//  if(getPreset() == 'U' || getPreset() == 'H' || getPreset() == 'J')
//  {
//    setup();
//  }
  
  // This function call takes care of movement
  chairOp(presetInput);

  // This function call takes care of data gathering and calculations
  dataCalc();
  
/**
 * Function called to write data to serial port if session has ended.
 * <p>
 * Note: Former "writeToGUI" method - this NEEDS to stay here
 * @param numCollisions number of times sensors detect "collision"
 * @param closestDistance closest distance (in cm?) WITHOUT colliding
 */
  if(Serial.available() > 0)
  {
//    if(getPreset() == 'D')
//    {
      String results = "";
      doneRecording = Serial.read();
//    }

    if(doneRecording == 'S')
    {
      numCollisions = 0;
      closestDistance = 0;
    }
    
    // Command signal for "done recording"
    if(doneRecording == 'D')
    {
      /**
       * Concatenates results to string, separated by comma
       * to make it readable when data reaches Processing end
       */
      results = "Number of collisions: " + String(numCollisions) + ",Closest distance (cm): " + String(closestDistance);

      // This sends info to Processing
      Serial.println(results);
    }

    // This allows user to go back and change the preset mid-session if they want
    else if(doneRecording == 'X')
    {
      setup();
    }
  }
}
/**
 * Function called if user chooses unidirectional joystick. Delete if unused
 */
void uniJoystickSetup()
{
    pinMode(LED_BUILTIN, OUTPUT);

}

/**
 * Function called if user chooses head switches. Delete if unused
 */
void headSwitchSetup()
{
    pinMode(LED_BUILTIN, OUTPUT);
        digitalWrite(LED_BUILTIN, LOW);
}

/**
 * Function called if user chooses regular joystick. Delete if unused
 */
void regJoystickSetup()
{
    pinMode(LED_BUILTIN, OUTPUT);
        digitalWrite(LED_BUILTIN, LOW);
}

/**
 * Function called to control movement of chair - this will depend on
 * what preset was chosen.
 */
void chairOp(char presetInput)
{
  // Movement code if user decided on regular joystick
  // For this, onboard LED blinks at 1sec intervals
  if(presetInput == 'J')
  {
      digitalWrite(LED_BUILTIN, HIGH);   // turn the LED on (HIGH is the voltage level)
      delay(1000);                       // wait for a second
      digitalWrite(LED_BUILTIN, LOW);    // turn the LED off by making the voltage LOW
      delay(1000);                       // wait for a second
  }
  
  // Movement code if user decided on unidirectional joystick
  // For this, onboard LED blinks at 2sec intervals
  if(presetInput == 'U')
  {
      digitalWrite(LED_BUILTIN, HIGH);   // turn the LED on (HIGH is the voltage level)
      delay(2000);                       // wait for a second
      digitalWrite(LED_BUILTIN, LOW);    // turn the LED off by making the voltage LOW
      delay(2000);                       // wait for a second
  }

  // Movement code if user decided on head switches
  // For this, onboard LED blinks at 3sec intervals
  else if(presetInput == 'H')
  {
      digitalWrite(LED_BUILTIN, HIGH);   // turn the LED on (HIGH is the voltage level)
      delay(3000);                       // wait for a second
      digitalWrite(LED_BUILTIN, LOW);    // turn the LED off by making the voltage LOW
      delay(3000);                       // wait for a second
  }
}

/**
 * Function called to calculate metrics that we plan to present Krista
 * at the end of each session. Uses sensor data.
 */
void dataCalc()
{
  /**
   * Make sure to use numCollisions and closestDistance variables to store
   * results in - if you change the names, you'll have to do that several
   * times throughout the code :P
   */
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);

  duration = pulseIn(echoPin, HIGH);
  distance = (duration*.0343)/2;
  if(distance < 10)
  {
    numCollisions++;
    closestDistance = 0;
  }

  else if(distance > 10 && distance < 300)
  {
      if(distance < closestDistance || closestDistance == 0)
      {
        closestDistance = distance;
      }
  }
  delay(100);
}
//
///**
// * Function to get preset from user, ESSENTIAL for full functionality
// */
//char getPreset()
//{
//  if(Serial.available())
//  {
//    if(Serial.read() == 'J' || Serial.read() == 'U' || Serial.read() == 'H')
//    {
//      presetInput = Serial.read();
//    }
//  }
//  return presetInput;
//}

